<?php

namespace core\log;

/**
 * Type containing a list of tags
 */
class TagsType implements \ArrayAccess, \Iterator, \Countable
{

	/**
	 * current position in the array
	 *
	 * @var \int
	 */
	protected int $position;

	/**
	 * tags inside this object
	 *
	 * @var \array[\int, \core\log\Tag]
	 */
	protected array $values;

	/**
	 * keys associated to the tags array
	 *
	 * @var \array[\int, \int]
	 */
	protected array $keys;

	/**
	 * TagsType's constructor
	 *
	 * @param \array[\int, \string] $values Tags
	 *
	 * @throws \core\type\TypingException
	*/
	public function __construct(array $values)
	{
		$this->position = 0;

		$this->values = [];
		foreach ($values as $value)
		{
			$this->values[] = $this->build($value);
		}
		$this->keys = \array_keys($this->values);
	}

	public function build(string $tag_name) : \core\log\Tag
	{
		$tag = null;

		$tag_name = \ucwords($tag_name);
		$tag_name = \str_replace(' ', '', $tag_name);

		$vanilla = '\\core\\log\\tags\\' . $tag_name;

		if (\class_exists($vanilla))
		{
			$tag = new $vanilla();
		}
		else
		{
			$plugin = \core\plugin\PluginManager::fetch_class($vanilla);

			if (!\is_null($plugin) && \class_exists($plugin))
			{
				$tag = new $plugin();
			}
			else
			{
				throw new \core\config\ConfigException(\core\substitute(
					_('the tag {tag} does not exist'),
					array('tag' => $tag_name),
				), 1);
			}
		}

		if (!($tag instanceof \core\log\Tag))
		{
			throw new \core\config\ConfigException(\core\substitute(
				_('tag {tag} does not inherit Base tag class'),
				array('tag' => $tag_name),
			), 2);
		}

		return $tag;
	}

	/**
	 * Get the number of tags
	 *
	 * Used by the \Countable interface
	 *
	 * @return \int number of tags
	 */
	public function count() : int
	{
		return \count($this->values);
	}

	/**
	 * Reset the position to 0
	 *
	 * Used by the \Iterator interface
	 */
	public function rewind() : void
	{
		$this->position = 0;
	}

	/**
	 * Get the current tag
	 *
	 * Used by the \Iterator interface
	 *
	 * @return \core\log\Tag current tag
	 */
	public function current() : \core\log\Tag
	{
		return $this->values[$this->keys[$this->position]];
	}

	/**
	 * Get the current key
	 *
	 * Used by the \Iterator interface
	 *
	 * @return \int current key
	 */
	public function key() : int
	{
		return $this->keys[$this->position];
	}

	/**
	 * Increase the position by one
	 *
	 * Used by the \Iterator interface
	 */
	public function next() : void
	{
		$this->position += 1;
	}

	/**
	 * Check if the current position is valid
	 *
	 * Used by the \Iterator interface
	 *
	 * @return \bool
	 */
	public function valid() : bool
	{
		return isset($this->keys[$this->position]);
	}

	/**
	 * Set a tag in the list of tags
	 *
	 * Used by the \ArrayAccess interface
	 *
	 * @param \int | \null $offset Key of the tag to set
	 *
	 * @param \core\log\Tag tag to set
	 */
	public function offsetSet(mixed $offset, mixed $value) : void
	{
		$tag = $this->build($value);
		if (\is_null($offset))
		{
			$this->values[] = $tag;
			$this->keys[] = \array_key_last($this->values);
		}
		else
		{
			if (!\in_array($offset, $this->keys))
			{
				$this->keys[] = $offset;
			}
			$this->values[$offset] = $tag;
		}
	}

	/**
	 * Check if the offset exist
	 *
	 * Used by the \ArrayAccess interface
	 *
	 * @param \int $offset The offset to check the existence of
	 *
	 * @return \bool
	 */
	public function offsetExists(mixed $offset) : bool
	{
		return isset($this->values[$offset]);
	}

	/**
	 * Unset a tag by key
	 *
	 * Used by the \ArrayAccess interface
	 *
	 * @param \int $offset Offset of the tag to unset
	 */
	public function offsetUnset(mixed $offset) : void
	{
		unset($this->values[$offset]);
		unse($this->keys[\array_search($offset, $this->keys)]);
		$this->keys = \array_values($this->keys);
	}

	/**
	 * Get a tag by key
	 *
	 * Used by the \ArrayAccess interface
	 *
	 * @param \int $offset Key associated to the tag
	 *
	 * @return \string The wanted tag
	 */
	public function offsetGet(mixed $offset) : string
	{
		return isset($this->values[$offset]) ? $this->values[$offset] : null;
	}
}

?>
